// Programming Techniques - Project
// Ken Webb
// January 5, 2003
// Application.h: interface for the Application class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_APPLICATION_H__4E70204A_F0D4_45A2_B809_4A9E3A3D1A28__INCLUDED_)
#define AFX_APPLICATION_H__4E70204A_F0D4_45A2_B809_4A9E3A3D1A28__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/**************************************************************************************
                                        Application

Simple concrete application, conceived of as three phases:
1. initialize
2. process
3. wrapup

To write a more complex application, use this class as an example of how to
use the other classes in this project.
**************************************************************************************/

#include "TreeNodeFactory.h"
#include "InheritanceHierarchy.h"
#include "ContainmentHierarchy.h"

// Maximum size of a file name
const int MaxFileNameSize = 50;

class Application  
{
public:
	// constructor and destructor
	Application();
	virtual ~Application();

	void initialize( char *configFileName ); // initialize
	void process( void );                    // process
	void wrapup( void );                     // wrapup

protected:
	// protected data members
	TreeNodeFactory *factory;          // factory
	InheritanceHierarchy *inherHier;   // inheritance hierarchy
	ContainmentHierarchy *containHier; // containment hierarchy
	BioEntityClass *becRoot;           // root of BioEntityClass node hierarchy
	BioEntity *root;                   // root of BioEntity node hierarchy
	int maxProcessLoops; // number of times to loop within process()

	// filenames
	char bioEntityClassNamesFile[MaxFileNameSize];
	char inheritanceHierarchyDataFile[MaxFileNameSize];
	char containmentHierarchyDataFile[MaxFileNameSize];

	// config values
	int maxBioEntities, maxBioEntityClasses;
	int bevalGeneMin, bevalGeneMax, bevalPheneMin, bevalPheneMax;

	// protected member functions
	void readConfigFromFile( char *fileName );
};

#endif // !defined(AFX_APPLICATION_H__4E70204A_F0D4_45A2_B809_4A9E3A3D1A28__INCLUDED_)
