// Programming Techniques - Project
// Ken Webb
// January 6, 2004
// ContainmentHierarchy.cpp: implementation of the ContainmentHierarchy class.
//
//////////////////////////////////////////////////////////////////////

#include "ContainmentHierarchy.h"
#include "Msg.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ContainmentHierarchy::ContainmentHierarchy()
{
	rootNode = NULL;
	factory = NULL;
	inherHier = NULL;
	nextId = 0;
}

ContainmentHierarchy::~ContainmentHierarchy()
{
	if (rootNode != NULL) {
		// Should return all nodes to factory using post order recursive
		// descent starting with root.
	}
}

////////////////////////////////////////////////////////////////////
// setFactory - set pointer to singleton TreeNodeFactory instance
////////////////////////////////////////////////////////////////////
void ContainmentHierarchy::setFactory( TreeNodeFactory* f )
{ factory = f; }

/////////////////////////////////////////////////////////////////////////////
// setInheritanceHierarchy - set pointer to InheritanceHierarchy instance
/////////////////////////////////////////////////////////////////////////////
void ContainmentHierarchy::setInheritanceHierarchy( InheritanceHierarchy* ih )
{ inherHier = ih; }

///////////////////////////////////////////////////////////////
// getRootNode - get root node of the containment hierarchy
///////////////////////////////////////////////////////////////
BioEntity* ContainmentHierarchy::getRootNode( void )
{ return rootNode; }

////////////////////////////////////////////////////////////////////////
// populateFromFile - populate the tree by reading items from a file
// multiplicity is the number of instances of that type to create
////////////////////////////////////////////////////////////////////////
void ContainmentHierarchy::populateFromFile( char *fileName )
{
	BioEntity *oldNode = NULL;
	BioEntity *newNode = NULL;
	int oldLevel = 0;
	int newLevel = 0;
	int becId = 0; // BioEntityClass id from file
	int multiplicity = 1;

	FILE *fp = openFile( fileName );

	// handle root node
	int rc = getData( fp, &oldLevel, &becId, &multiplicity );
	if (rc != EOF) {
		// multiplicity of root can only be 1
		rootNode = factory->getBioEntityNode();
		oldNode = rootNode;
		oldNode->setId( nextId++ );
		oldNode->setBec( becId, inherHier->getClassNode( becId ) );
	}

	// handle non-root nodes
	rc = getData( fp, &newLevel, &becId, &multiplicity );
	while (rc != EOF) {
		newNode = factory->getBioEntityNode();
		newNode->setId( nextId++ );
		newNode->setBec( becId, inherHier->getClassNode( becId ) );
		if (newLevel > oldLevel) { // oldNode is parent of newNode
			newNode->setParentChildLinks( oldNode );
		}
		else {
			while (newLevel < oldLevel) { // have reached end of sibling chain
				oldLevel--;
				oldNode = (BioEntity*)oldNode->getParent();
			}
			newNode->setParentSiblingLinks( oldNode );
		}
		oldLevel = newLevel;
		oldNode = newNode;
		if (multiplicity == 1) // handle multiplicity
			rc = getData( fp, &newLevel, &becId, &multiplicity );
		else
			multiplicity--;
	}

	closeFile( fp );
}

/////////////////////////////////
// openFile - open input file
/////////////////////////////////
FILE* ContainmentHierarchy::openFile( char *fileName )
{
	FILE* fp = fopen( fileName, "r" );
	if (fp == NULL) {
		if (Msg::errorM) printf( "Can't open file %s\n", fileName );
		exit (0);
	}
	return fp;
}

////////////////////////////
// closeFile - close file
////////////////////////////
void ContainmentHierarchy::closeFile( FILE *fp )
{
	fclose( fp );
}

///////////////////////////////////////////////////////////////////////
// Get level and id in inheritence hierarchy (from file)
// ex: 03,207,7	// Mitochondrion (only first 3 fields are processed)
///////////////////////////////////////////////////////////////////////
int ContainmentHierarchy::getData( FILE *fp, int *level, int *id, int *multiplicity )
{
	int rc = 1; // a value other than EOF
	const int SizeBuff = 100;
	static char buffer[SizeBuff];
	if (fgets(buffer, SizeBuff, fp) != NULL) {
		if (sscanf( buffer, "%d,%d,%d", level, id, multiplicity ) != 3) {
			if (Msg::errorM) printf("ContainmentHierarchy: error reading from file\n");
			exit(0);
		}
	}
	else {
		rc = EOF;
	}
	return rc;
}
