// Programming Techniques - Project
// Ken Webb
// January 5, 2004
// ContainmentHierarchy.h: interface for the ContainmentHierarchy class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CONTAINMENTHIERARCHY_H__ADD0C76C_BE53_4646_B925_79D4EC47AE6A__INCLUDED_)
#define AFX_CONTAINMENTHIERARCHY_H__ADD0C76C_BE53_4646_B925_79D4EC47AE6A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/**************************************************************************************
                                        ContainmentHierarchy

A ContainmentHierarchy represents a hierarchy (tree) of BioEntity nodes contained
within other BioEntity nodes.
It encapsulates:
  the root node of a BioEntity hierarchy (rootNode).
  the next BioEntity id to be allocated (nextId).
  pointers to a BioEntity factory, and to an associated inheritance hierarchy.

The primary role of the ContainmentHierarchy is to read a BioEntity hierarchy in from a
file, and store the nodes in a tree.

**************************************************************************************/

#include "BioEntity.h"
#include "TreeNodeFactory.h"
#include "InheritanceHierarchy.h"

class ContainmentHierarchy  
{
public:
	// constructor and destructor
	ContainmentHierarchy();
	virtual ~ContainmentHierarchy();

	// get and set functions
	void setFactory( TreeNodeFactory* f );
	void setInheritanceHierarchy( InheritanceHierarchy* ih );
	BioEntity* getRootNode( void );

	// Populate a BioEntity tree by reading in nodes from a file
	void populateFromFile( char *fileName );

private:
	// private data members
	BioEntity* rootNode; // root of the encapsulated tree
	TreeNodeFactory* factory; // factory that allocates memory for nodes
	InheritanceHierarchy* inherHier; // corresponds to this containment hierarchy
	int nextId; // next Id to be allocated

	// private member functions
	// open a file
	FILE* openFile( char *fileName );

	// close a file
	void closeFile( FILE *fp );

	// get one line of inheritance hierarchy data from a file
	int getData( FILE *fp, int *level, int *id, int *multiplicity );
};

#endif // !defined(AFX_CONTAINMENTHIERARCHY_H__ADD0C76C_BE53_4646_B925_79D4EC47AE6A__INCLUDED_)
