// Programming Techniques - Project
// Ken Webb
// January 5, 2004
// InheritanceHierarchy.h: interface for the InheritanceHierarchy class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_INHERITANCEHIERARCHY_H__2DC79410_5972_4643_9291_DEB6CC544212__INCLUDED_)
#define AFX_INHERITANCEHIERARCHY_H__2DC79410_5972_4643_9291_DEB6CC544212__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/**************************************************************************************
                                        InheritanceHierarchy

An InheritanceHierarchy represents a hierarchy (tree) of BioEntityClass nodes that
inherit from other BioEntityClass nodes.
It encapsulates:
  the root node of a BioEntityClass hierarchy (rootNode).
  a pointer to a BioEntity factory.
  an array of pointers to BioEntityClass names (classNames).
  an array of pointers to BioEntityClass instances (classPtr)
    (a redundant structure that allows O(1) time access,
	 rather than having to search the tree).

The primary role of the InheritanceHierarchy is:
  to read BioEntityClass names in from a file, and store these in an array.
  to read a BioEntityClass hierarchy in from a file, and store the nodes in a tree.

**************************************************************************************/

#include "BioEntityClass.h"
#include "TreeNodeFactory.h"

// Max number of BioEntityClass names
// should be passed in as a parameter, and data structures dynamically created
// but it works fine for now
const int MaxBeClasses = 500;

class InheritanceHierarchy  
{
public:
	// constructor and destructor
	InheritanceHierarchy();
	virtual ~InheritanceHierarchy();

	// get and set functions
	void setFactory( TreeNodeFactory* f );
	BioEntityClass* getClassNode( int beClassId );
	BioEntityClass* getRootNode( void );

	// Populate a BioEntityClass tree by reading in nodes from a file
	void populateFromFile( char *fileName );

	// Populate the names array by reading in from a file
	void populateNamesFromFile( char *fileName );

	// Print the complete list of BioEntityClass names
	void printBioEntityClassNames( void );

private:
	// private data members
	BioEntityClass* rootNode; // root of the encapsulated tree
	TreeNodeFactory* factory; // factory that allocates memory for nodes
	char *className[MaxBeClasses]; // array of BioEntityClass names
	BioEntityClass* classPtr[MaxBeClasses]; // array of pointers to BioEntityClasses

	// private member functions
	// open a file
	FILE* openFile( char *fileName );

	// close a file
	void closeFile( FILE *fp );

	// get one line of inheritance hierarchy data from a file
	int getData( FILE *fp, int *level, int *id );

	// get one line of BioEntityClass name data from a file
	int getName( FILE *fp, char *buffer, int sizeBuff );
};

#endif // !defined(AFX_INHERITANCEHIERARCHY_H__2DC79410_5972_4643_9291_DEB6CC544212__INCLUDED_)
