// Programming Techniques - Project
// Ken Webb
// January 5, 2004
// TreeNode.cpp: implementation of the TreeNode class.
//
//////////////////////////////////////////////////////////////////////

#include "TreeNode.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TreeNode::TreeNode()
{
	initialize();
}

TreeNode::~TreeNode()
{
	// recursively delete children and siblings
	if (leftChild != NULL)
		delete leftChild;
	if (rightSibling != NULL)
		delete rightSibling;
}

/////////////////////////////////
// get and set functions
/////////////////////////////////
TreeNode* TreeNode::getParent( void )       { return parent; }
TreeNode* TreeNode::getLeftChild( void )    { return leftChild; }
TreeNode* TreeNode::getRightSibling( void ) { return rightSibling; }
void TreeNode::setParent( TreeNode* treeNode )     { parent = treeNode; }
void TreeNode::setLeftChild( TreeNode* treeNode )  { leftChild = treeNode; }
void TreeNode::setRightSibling( TreeNode* treeNode ) { rightSibling = treeNode; }

/////////////////////////////////////////////////////////
// setParentChildLinks
// call this function on the leftChild of a parent
/////////////////////////////////////////////////////////
void TreeNode::setParentChildLinks( TreeNode* parent )
{
	setParent( parent );
	parent->setLeftChild( this );
}

/////////////////////////////////////////////////////////////////////////////
// setParentSiblingLinks
// call this function on the node that is pointed to by the leftSibling
/////////////////////////////////////////////////////////////////////////////
void TreeNode::setParentSiblingLinks( TreeNode* leftSibling )
{
	setParent( leftSibling->getParent() ); // leftSibling already has parent
	leftSibling->setRightSibling( this );
}

// standard tree utility functions

/////////////////////////////////////////////////////////////////////
// height - return height of the tree starting from this node
// The height is computed by traversing the leftChild nodes only
/////////////////////////////////////////////////////////////////////
int TreeNode::height( void )
{
	if ( leftChild == NULL )
		return 0;
	else {
		return ( 1 + leftChild->height() );
	}
}

////////////////////////////////////////////////////////////////////////////////
// treeSize - return size of a tree
// total number of nodes in the tree, starting from and including this node
////////////////////////////////////////////////////////////////////////////////
int TreeNode::treeSize( void )
{
	int lSize = 0;
	int rSize = 0;
	if ( leftChild != NULL )
		lSize = leftChild->treeSize();
	if ( rightSibling != NULL )
		rSize = rightSibling->treeSize();
	return ( 1 + lSize + rSize );
}

//////////////////////////////////////////////////////////////////
// isRoot - return whether this node is the root of a tree
//////////////////////////////////////////////////////////////////
bool TreeNode::isRoot( void )
{
	if (parent == NULL)
		return true;
	else
		return false;
}

//////////////////////////////////////////////////////////////////////////////
// isExternal - return whether this is an external node (no child or sibling)
//////////////////////////////////////////////////////////////////////////////
bool TreeNode::isExternal( void )
{
	return ( (leftChild == NULL) && (rightSibling == NULL) );
}

/////////////////////////////////////////////////////////////////
// isInternal - return whether this tree is an internal node
//   (at least one child or sibling)
/////////////////////////////////////////////////////////////////
bool TreeNode::isInternal( void )
{
	return ( (leftChild != NULL) || (rightSibling != NULL) );
}

// tree traversal print functions that MAY be overridden in concrete subclasses

////////////////////
// preOrderPrint
////////////////////
void TreeNode::preOrderPrint( int level )
{
	printf( "." );
	if (leftChild != NULL)
		leftChild->preOrderPrint();
	if (rightSibling != NULL)
		rightSibling->preOrderPrint();
}

////////////////////
// inOrderPrint
////////////////////
void TreeNode::inOrderPrint( int level )
{
	if (leftChild != NULL)
		leftChild->inOrderPrint();
	printf( "," );
	if (rightSibling != NULL)
		rightSibling->inOrderPrint();
}

/////////////////////
// postOrderPrint
/////////////////////
void TreeNode::postOrderPrint( int level )
{
	if (leftChild != NULL)
		leftChild->postOrderPrint();
	if (rightSibling != NULL)
		rightSibling->postOrderPrint();
	printf( ";" );
}

///////////////////
// initialize
///////////////////
void TreeNode::initialize( void )
{
	parent       = NULL;
	leftChild    = NULL;
	rightSibling = NULL;
}

/////////////////////////////////////////////////////////////////////////////////////
// configure - perform some action, typically once at start up; MAY be overridden
/////////////////////////////////////////////////////////////////////////////////////
void TreeNode::configure( void )
{ printf( "TreeNode: configure()\n" ); }

///////////////////////////////////////////////////////////////
// act - perform some repetitive action; MAY be overridden
///////////////////////////////////////////////////////////////
void TreeNode::act( void )
{ printf( "TreeNode: act()\n" ); }
