// Programming Techniques - Project
// Ken Webb
// January 5, 2004
// TreeNode.h: interface for the TreeNode class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TREENODE_H__FC963AA5_975D_496D_A2E3_EA0ED08C9EE6__INCLUDED_)
#define AFX_TREENODE_H__FC963AA5_975D_496D_A2E3_EA0ED08C9EE6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/**************************************************************************************
                                        TreeNode

This is an abstract superclass from which no instances can be created.
To use this class, you must first create a concrete subclass.

Each instance of a TreeNode subclass encapulates pointers to other instances of TreeNode:
  parent, leftChild, rightSibling

Example: (^ and v represent up and down arrow heads)

         one
          ^
         / \
        /   \
       v     \
      two-->three
       ^
      /
     /
    v
  four

In this example:
- one has no parent and no rightSibling, but does have a leftChild (= two).
- two has a parent (one), a leftChild (four), and a rightSibling (three).
- three has a parent (one).
- four has a parent (two).
- one is the root node.
- one has treeSize() of 4.
- one has height() of 2.

**************************************************************************************/

// These are included here as a convenience.
// They should instead be placed in each .cpp or .h file that needs them.
#include <stdio.h>
#include <stdlib.h>

// nodes for use in a binary tree implementation of an n-ary tree
// an abstract pure virtual class
// does tree stuff only; no application contents
class TreeNode  
{
public:
	// constructors and destructors
	TreeNode();
	virtual ~TreeNode();

	// get and set functions
	TreeNode* getParent( void );       // parent
	void setParent( TreeNode* treeNode );
	TreeNode* getLeftChild( void );    // leftmost child
	void setLeftChild( TreeNode* treeNode );
	TreeNode* getRightSibling( void ); // right sibling
	void setRightSibling( TreeNode* treeNode );

	// The following are two convenience functions to set related links.
	// Use these instead of the above three set functions to help avoid making errors.

	// equivalent to:
	//   node->setParent( parent );
	//   parent->setLeftChild( node );
	void setParentChildLinks( TreeNode* parent );

	// equivalent to:
	//   node->setParent( leftSibling->getParent() );
	//   leftSibling->setRightSibling( node );
	void setParentSiblingLinks( TreeNode* leftSibling );

	// standard tree utility functions
	int height( void );   // get number of levels in hierarchical tree
	int treeSize( void ); // get number of nodes in tree
	bool isRoot( void );  // is this the root node of the tree (does it have a parent)
	bool isExternal( void ); // is this an external node (no child or siblings)
	bool isInternal( void ); // is this an internal node (has child and/or siblings)

	// tree traversal functions that MUST be overridden in concrete subclasses
	// allow concrete instances to perform some application specific function
	virtual void preOrder( void ) = 0;
	virtual void inOrder( void ) = 0;
	virtual void postOrder( void ) = 0;

	// tree traversal print functions that MAY be overridden in concrete subclasses
	virtual void preOrderPrint( int level = 0 );
	virtual void inOrderPrint( int level = 0 );
	virtual void postOrderPrint( int level = 0 );

	// initialize
	void initialize( void );

	// Perform some action, typically once at start up; MAY be overridden
	virtual void configure( void );

	// Perform some repetitive action; MAY be overridden
	virtual void act( void );

protected:
	// private member variables
	TreeNode *parent;       // parent, or NULL if root
	TreeNode *leftChild;    // leftmost child, or NULL
	TreeNode *rightSibling; // rightmost sibling, or NULL
};

#endif // !defined(AFX_TREENODE_H__FC963AA5_975D_496D_A2E3_EA0ED08C9EE6__INCLUDED_)
