// Programming Techniques - Project
// Ken Webb
// January 5, 2004
// TreeNodePlus_Main_Test1.cpp:

//////////////////////////////////////////////////////////////////////////
// main()
// To run the test application, from a console window run:
//    TestNodePlus 1
// 
// Unit test all the classes.
// 
// This file contains the total of all unit test cases I have devised,
// while testing each new class as it was developed.
//////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include "BioEntity.h"
#include "BioEntityClass.h"
#include "TreeNodeFactory.h"
#include "Queue.h"
#include "TypeData.h"
#include "InheritanceHierarchy.h"
#include "ContainmentHierarchy.h"
#include "Application.h"
#include "Msg.h"

// C Function Prototypes
void test1( void );
void test2( void );
void test3( void );
void test4( void );
void qTest1( void );
void factoryTest1( void );
void factoryTest2( void );
void factoryTest3( void );
void bioEntityClassTest1( void );
void bioEntityClassTest2( void );
void inheritanceHierTest1( void );
void inheritanceHierTest2( void );
void containmentHierTest1( void );
void applicationTest1( void );

// main function
int main( int argc, char *argv[] )
{
	if (argc < 2) {
		printf( "usage: TreeNodePlus n (where n is between 1 and 14)\n" );
		printf( "       TreeNodePlus 1 (to run the concrete application\n" );
		exit (0);
	}
	// printf( "%s\n", argv[1] );
	int testNum = atoi( argv[1] );

	if (testNum == 1) applicationTest1();
	else if (testNum == 2) qTest1();
	else if (testNum == 3) factoryTest1();
	else if (testNum == 4) factoryTest2();
	else if (testNum == 5) factoryTest3();
	else if (testNum == 6) bioEntityClassTest1();
	else if (testNum == 7) bioEntityClassTest2();
	else if (testNum == 8) inheritanceHierTest1();
	else if (testNum == 9) inheritanceHierTest2();
	else if (testNum == 10) containmentHierTest1();
	else if (testNum == 11) test1();
	else if (testNum == 12) test2();
	else if (testNum == 13) test3();
	else if (testNum == 14) test4();
	return 0;
}

// Application Test 1
void applicationTest1( void )
{
	if (Msg::infoM) printf( "Application Test 1\n" );

	Application *app;
	app = new Application();
	app->initialize( "..\\ConfigFile.txt" );
	app->process();
	app->wrapup();
	delete app;
}

// Containment Hierarchy Test 1
void containmentHierTest1( void )
{
	printf( "ContainmentHierarchy Test 1\n" );

	TreeNodeFactory *factory;
	factory = new TreeNodeFactory();

	InheritanceHierarchy *inherHier;
	inherHier = new InheritanceHierarchy;
	inherHier->setFactory( factory );
	inherHier->populateNamesFromFile( "..\\TypeData.h" );
	// inherHier->printBioEntityClassNames();
	inherHier->populateFromFile( "..\\CellData_2.txt" );

	ContainmentHierarchy *containHier;
	containHier = new ContainmentHierarchy;

	containHier->setFactory( factory );
	containHier->setInheritanceHierarchy( inherHier );
	// containHier->populateFromFile( "..\\CellContainData_1.txt" );
	containHier->populateFromFile( "..\\CellContainData_2.txt" );

	BioEntityClass::seedRandomNumberGenerator();
	BioEntity *root;
	root = containHier->getRootNode();
	root->configure();
	printf( "\n" );
	// printf( "Pre-order Print [Level, BioEntityId, BioEntityClassId, BioEntityName]\n" );
	printf( "Pre-order Print . . .\n" );
	root->preOrderPrint();
	// printf( "\n\n" );
	// printf( "In-order Print [Level, BioEntityId, BioEntityClassId, BioEntityName]\n" );
	// root->inOrderPrint();
	// printf( "\n\n" );
	// printf( "Post-order Print [Level, BioEntityId, BioEntityClassId, BioEntityName]\n" );
	// root->postOrderPrint();
	printf( "\n" );

	printf( "\n" );
	for (int j = 0; j < 10; j++) {
		root->act();
		printf( "\n" );
	}
	printf( "\n" );

	delete containHier;
	delete inherHier;
	delete factory;
}

// Inheritance Hierarchy Test 1
void inheritanceHierTest1( void )
{
	printf( "InheritanceHierarchy Test 1\n" );
	TreeNodeFactory *factory;
	factory = new TreeNodeFactory();
	InheritanceHierarchy *inherHier;
	inherHier = new InheritanceHierarchy;

	inherHier->setFactory( factory );

	inherHier->populateNamesFromFile( "..\\TypeData.h" );
	inherHier->printBioEntityClassNames();

	// inherHier->populateFromFile( "..\\TestBecData_1.txt" );
	// inherHier->populateFromFile( "..\\TestBecData_2.txt" );
	// inherHier->populateFromFile( "..\\CellData_1.txt" );
	inherHier->populateFromFile( "..\\CellData_2.txt" );

	BioEntityClass *root;
	root = inherHier->getRootNode();
	printf( "\n" );
	printf( "Pre-order Print [Level, BioEntityClass id, BioEntityClass name]\n" );
	root->preOrderPrint();
	printf( "\n\n" );
	printf( "In-order Print [Level, BioEntityClass id, BioEntityClass name]\n" );
	root->inOrderPrint();
	printf( "\n\n" );
	printf( "Post-order Print [Level, BioEntityClass id, BioEntityClass name]\n" );
	root->postOrderPrint();
	printf( "\n" );

	printf( "\n" );
	delete inherHier;
	delete factory;
}

// Inheritance Hierarchy Test 2
void inheritanceHierTest2( void )
{
	printf( "InheritanceHierarchy Test 2\n" );
	InheritanceHierarchy *inherHier;
	inherHier = new InheritanceHierarchy;

	inherHier->populateNamesFromFile( "..\\TypeData.h" );
	inherHier->printBioEntityClassNames();
}

// BioEntityClass Test 1
void bioEntityClassTest1( void )
{
	printf( "BioEntityClass Test 1\n" );
	TreeNodeFactory *factory;
	factory = new TreeNodeFactory();
	BioEntityClass *beClass;
	beClass = factory->getBioEntityClassNode();
	printf( "Height: %d\n", beClass->height() );
	printf( "\n" );

	beClass->setId( EnzymeTDE );
	beClass->setName( "Enzyme" );
	char* instruct = beClass->getConfigurationInstructions();

	printf( "BioEntityClass Id: %d\n", beClass->getId() );
	printf( "Name: %s [%s]\n", beClass->getName(), instruct );

	factory->returnBioEntityClassNode( beClass );
	printf( "\n" );
	delete factory;
}

// BioEntityClass Test 2
void bioEntityClassTest2( void )
{
	printf( "BioEntityClass Test 2\n" );
	TreeNodeFactory *factory;
	factory = new TreeNodeFactory();
	BioEntityClass *beClass;
	beClass = factory->getBioEntityClassNode();
	BioEntityClass::seedRandomNumberGenerator();
	//int rInt = 0;
	//for (int i = 0; i < 100; i++) {
	//	rInt = beClass->getRandomInt( -10, 11 );
	//	printf( "%d ", rInt );
	//}

	beClass->setId( EnzymeTDE );
	beClass->setName( "Enzyme" );
	char* instruct = beClass->getConfigurationInstructions();
	printf( "BioEntityClass Id: %d\n", beClass->getId() );
	printf( "Name: %s [%s]\n", beClass->getName(), instruct );

	factory->returnBioEntityClassNode( beClass );
	printf( "\n" );
	delete factory;
}

// Queue Test 1
void qTest1( void )
{
	int i;
	int MaxQSize = 10;
	Queue *testQ;
	printf( "Queue Test 1\n" );
	testQ = new Queue( MaxQSize ); // Queue with 10 slots

	printf( "Queue size: %d\n", testQ->getSize() );
	printf( "dequeue when empty: %x\n", testQ->dequeue() );

	testQ->enqueue( (void *)"hello world" );
	printf( "Queue size: %d\n", testQ->getSize() );
	printf( "dequeue when 1 item: %x\n", testQ->dequeue() );

	testQ->enqueue( (void *)"One" );
	testQ->enqueue( (void *)"Two" );
	printf( "Queue size: %d\n", testQ->getSize() );
	char *item = (char *)testQ->dequeue();
	while (item != Q_EMPTY) {
		printf( "%s\n", item );
		item = (char *)testQ->dequeue();
	}
	printf( "Queue size: %d\n", testQ->getSize() );

	int rc;
	printf( "enqueue return code: " );
	for (i = 0; i < MaxQSize + 2; i++) {
		rc = testQ->enqueue( (void *)"Testing" );
		printf( "%d ", rc );
	}
	printf( "\n" );
}

// TreeNodeFactory Test 1
void factoryTest1( void )
{
	printf( "Factory Test 1\n" );
	TreeNodeFactory *factory;
	factory = new TreeNodeFactory();
	BioEntity *root;
	root = factory->getBioEntityNode();
	printf( "Height: %d\n", root->height() );
	printf( "\n" );
	factory->returnBioEntityNode( root );
	printf( "\n" );
	delete factory;
}

// TreeNodeFactory Test 2 - BioEntity nodes
void factoryTest2( void )
{
	printf( "Factory Test 2 - BioEntity ...\n" );
	BioEntity *root, *lChild, *rSibling;
	TreeNodeFactory *factory;
	factory = new TreeNodeFactory();

	root = factory->getBioEntityNode();

	lChild = factory->getBioEntityNode();
	lChild->setParent( root );
	root->setLeftChild( lChild );

	// strictly speaking, root shouldn't have any siblings
	rSibling = factory->getBioEntityNode();
	rSibling->setParent( root );
	root->setRightSibling( rSibling );

	root->preOrderPrint(); printf( "\n" );
	root->inOrderPrint(); printf( "\n" );
	root->postOrderPrint(); printf( "\n" );

	root->getParent();
	root->getLeftChild();
	root->getRightSibling();
	root->preOrder();
	root->inOrder();
	root->postOrder();

	printf( "Height: %d\n", root->height() );

	// Test boolean functions for root
	printf( "isRoot(root): " );
	if (root->isRoot())
		printf( "true\n" );
	else
		printf( "false\n" );

	printf( "isExternal(root): " );
	if (root->isExternal())
		printf( "true\n" );
	else
		printf( "false\n" );

	printf( "isInternal(root): " );
	if (root->isInternal())
		printf( "true\n" );
	else
		printf( "false\n" );

	// Test boolean functions for leftChild
	printf( "isRoot(left): " );
	if (lChild->isRoot())
		printf( "true\n" );
	else
		printf( "false\n" );

	printf( "isExternal(left): " );
	if (lChild->isExternal())
		printf( "true\n" );
	else
		printf( "false\n" );

	printf( "isInternal(left): " );
	if (lChild->isInternal())
		printf( "true\n" );
	else
		printf( "false\n" );

	printf( "\n" );
	factory->returnBioEntityNode( root );
#if ALLOC_TYPE == ALLOC_STATIC
	// next 2 lines crash if use ALLOC_DYNAMIC
	factory->returnBioEntityNode( lChild );
	factory->returnBioEntityNode( rSibling );
#endif
	delete factory;
}

// TreeNodeFactory Test 3 - BioEntityClass nodes
void factoryTest3( void )
{
	printf( "Factory Test 3 - BioEntityClass ...\n" );
	BioEntityClass *root, *lChild, *rSibling;
	TreeNodeFactory *factory;
	factory = new TreeNodeFactory();

	root = factory->getBioEntityClassNode();

	lChild = factory->getBioEntityClassNode();
	lChild->setParent( root );
	root->setLeftChild( lChild );

	// strictly speaking, root shouldn't have any siblings
	rSibling = factory->getBioEntityClassNode();
	rSibling->setParent( root );
	root->setRightSibling( rSibling );

	root->preOrderPrint(); printf( "\n" );
	root->inOrderPrint(); printf( "\n" );
	root->postOrderPrint(); printf( "\n" );

	root->getParent();
	root->getLeftChild();
	root->getRightSibling();
	root->preOrder();
	root->inOrder();
	root->postOrder();

	printf( "Height: %d\n", root->height() );

	// Test boolean functions for root
	printf( "isRoot(root): " );
	if (root->isRoot())
		printf( "true\n" );
	else
		printf( "false\n" );

	printf( "isExternal(root): " );
	if (root->isExternal())
		printf( "true\n" );
	else
		printf( "false\n" );

	printf( "isInternal(root): " );
	if (root->isInternal())
		printf( "true\n" );
	else
		printf( "false\n" );

	// Test boolean functions for leftChild
	printf( "isRoot(left): " );
	if (lChild->isRoot())
		printf( "true\n" );
	else
		printf( "false\n" );

	printf( "isExternal(left): " );
	if (lChild->isExternal())
		printf( "true\n" );
	else
		printf( "false\n" );

	printf( "isInternal(left): " );
	if (lChild->isInternal())
		printf( "true\n" );
	else
		printf( "false\n" );

	printf( "\n" );
	factory->returnBioEntityClassNode( root );
#if ALLOC_TYPE == ALLOC_STATIC
	// next 2 lines crash if use ALLOC_DYNAMIC
	factory->returnBioEntityClassNode( lChild );
	factory->returnBioEntityClassNode( rSibling );
#endif
	delete factory;
}

// Test 1 - empty root
void test1( void )
{
	printf( "Test 1 - empty root ...\n" );
	BioEntity *root;
	root = new BioEntity();
	root->preOrderPrint(); printf( "\n" );
	root->inOrderPrint(); printf( "\n" );
	root->postOrderPrint(); printf( "\n" );
	printf( "Height: %d\n", root->height() );
	printf( "\n" );
	delete root;
}

// Test 2 - unit test all functions
void test2( void )
{
	printf( "Test 2 ...\n" );
	BioEntity *root, *lChild, *rSibling;

	root = new BioEntity();

	lChild = new BioEntity();
	lChild->setParent( root );
	root->setLeftChild( lChild );

	// strictly speaking, root shouldn't have any siblings
	rSibling = new BioEntity();
	rSibling->setParent( root );
	root->setRightSibling( rSibling );

	root->preOrderPrint(); printf( "\n" );
	root->inOrderPrint(); printf( "\n" );
	root->postOrderPrint(); printf( "\n" );

	root->getParent();
	root->getLeftChild();
	root->getRightSibling();
	root->preOrder();
	root->inOrder();
	root->postOrder();

	printf( "Height: %d\n", root->height() );

	// Test boolean functions for root
	printf( "isRoot(root): " );
	if (root->isRoot())
		printf( "true\n" );
	else
		printf( "false\n" );

	printf( "isExternal(root): " );
	if (root->isExternal())
		printf( "true\n" );
	else
		printf( "false\n" );

	printf( "isInternal(root): " );
	if (root->isInternal())
		printf( "true\n" );
	else
		printf( "false\n" );

	// Test boolean functions for leftChild
	printf( "isRoot(left): " );
	if (lChild->isRoot())
		printf( "true\n" );
	else
		printf( "false\n" );

	printf( "isExternal(left): " );
	if (lChild->isExternal())
		printf( "true\n" );
	else
		printf( "false\n" );

	printf( "isInternal(left): " );
	if (lChild->isInternal())
		printf( "true\n" );
	else
		printf( "false\n" );

	printf( "\n" );
	delete root; // recursive delete
}

// Test 3 - multiple siblings
void test3( void )
{
	printf( "Test 3 ...\n" );
	BioEntity *root, *lChild, *rSibling, *rSiblingNew;

	root = new BioEntity();

	lChild = new BioEntity();
	lChild->setParentChildLinks( root );

	rSibling = new BioEntity();
	rSibling->setParentSiblingLinks( lChild );
	
	for (int i = 0; i < 10; i++) {
		rSiblingNew = new BioEntity();
		rSiblingNew->setParentSiblingLinks( rSibling );
		rSibling = rSiblingNew;
	}
	
	printf( "Size: %d\n", root->treeSize() );
	printf( "Height: %d\n", root->height() );

	root->preOrderPrint(); printf( "\n" );
	root->inOrderPrint(); printf( "\n" );
	root->postOrderPrint(); printf( "\n" );

	printf( "\n" );
	delete root; // recursive delete
}

// Test 4 - multiple levels of children
void test4( void )
{
	printf( "Test 4 ...\n" );
	BioEntity *root, *lChild, *lChildNew, *rSibling;

	root = new BioEntity();

	lChild = new BioEntity();
	lChild->setParentChildLinks( root );

	rSibling = new BioEntity();
	rSibling->setParentSiblingLinks( lChild );
	
	for (int i = 0; i < 10; i++) {
		lChildNew = new BioEntity();
		lChildNew->setParentChildLinks( lChild );
		rSibling = new BioEntity();
		rSibling->setParentSiblingLinks( lChildNew );
	
		lChild = lChildNew;
	}

	printf( "Size: %d\n", root->treeSize() );
	printf( "Height: %d\n", root->height() );

	root->preOrderPrint(); printf( "\n" );
	root->inOrderPrint(); printf( "\n" );
	root->postOrderPrint(); printf( "\n" );

	printf( "\n" );
	delete root; // recursive delete
}
